@@ -120,6 +120,9 @@ CONST = {
     "optin_const": "OPTIN",
     "optout_const": "OPTOUT",
     "noloop_const": "NOLOOP",
+    "reset_const": "RESET",
+    "const_user": "USER",
+    "full_const": "FULL",
 }
 
 
@@ -165,6 +168,9 @@ MESSAGE = fr"(?P<message>{VALID_TOKEN})"
 CHANNEL = fr"(?P<channel>{VALID_TOKEN})"
 GROUP = fr"(?P<group>{VALID_TOKEN})"
 CONSUMER = fr"(?P<consumer>{VALID_TOKEN})"
+CATEGORYNAME = fr"(?P<categoryname>{VALID_TOKEN})"
+USERNAME = fr"(?P<username>{VALID_TOKEN})"
+RULE = fr"(?P<rule>{VALID_TOKEN})"
 BIT = r"(?P<bit>0|1)"
 FLOAT = fr"(?P<float>{_FLOAT})"
 LONGITUDE = fr"(?P<longitude>{_FLOAT})"
@@ -249,6 +255,7 @@ SHUTDOWN = fr"(?P<shutdown>{c('shutdown')})"
 SWITCH = fr"(?P<switch>{c('switch')})"
 ON_OFF = fr"(?P<on_off>{c('on_off')})"
 CONST_ID = fr"(?P<const_id>{c('const_id')})"
+CONST_USER = fr"(?P<const_user>{c('const_user')})"
 ADDR = fr"(?P<addr>{c('addr')})"
 SKIPME = fr"(?P<skipme>{c('skipme')})"
 YES = fr"(?P<yes>{c('yes')})"
@@ -301,6 +308,9 @@ OPTIN_CONST = fr"(?P<optin_const>{c('optin_const')})"
 OPTOUT_CONST = fr"(?P<optout_const>{c('optout_const')})"
 NOLOOP_CONST = fr"(?P<noloop_const>{c('noloop_const')})"
 
+RESET_CONST = fr"(?P<reset_const>{c('reset_const')})"
+FULL_CONST = fr"(?P<full_const>{c('full_const')})"
+
 command_grammar = compile(COMMAND)
 
 # Here are the core grammars, those are tokens after ``command``.
@@ -308,7 +318,7 @@ command_grammar = compile(COMMAND)
 # Then it's grammar here is r"\s+ key \s+ value \s*", we needn't add `command`
 # here because every syntaxes starts with `command` so we will prepend `command`
 # in get_command_grammar function.
-NEW_GRAMMAR = {
+GRAMMAR = {
     "command_key": fr"\s+ {KEY} \s*",
     "command_pattern": fr"\s+ {PATTERN} \s*",
     "command_georadiusbymember": fr"""
@@ -337,6 +347,7 @@ NEW_GRAMMAR = {
     "command_slot_slotsubcmd_nodex": fr"""
         \s+ {SLOT} \s+ {SLOTSUBCMD} (\s+ {NODE})? \s*""",
     "command_password": fr"\s+ {PASSWORD} \s*",
+    "command_usernamex_password": fr"(\s+ {USERNAME})? \s+ {PASSWORD} \s*",
     "command_message": fr"\s+ {MESSAGE} \s*",
     "command_messagex": fr"(\s+{MESSAGE})? \s*",
     "command_index": fr"\s+ {INDEX} \s*",
@@ -432,6 +443,7 @@ NEW_GRAMMAR = {
             (\s+ {ADDR} \s+ {IP_PORT})|
             (\s+ {CONST_ID} \s+ {CLIENTID})|
             (\s+ {TYPE_CONST} \s+ {CONNTYPE})|
+            (\s+ {CONST_USER} \s+ {USERNAME})|
             (\s+ {SKIPME} \s+ {YES})
         )+ \s*""",
     "command_migrate": fr"""\s+ {HOST} \s+ {PORT}
@@ -472,7 +484,10 @@ NEW_GRAMMAR = {
         (
             (\s+ {STREAM_CONSUMERS} \s+ {KEY} \s+ {GROUP})|
             (\s+ {STREAM_GROUPS} \s+ {KEY})|
-            (\s+ {STREAM} \s+ {KEY})|
+            (\s+ {STREAM} \s+ {KEY}
+                (\s+ {FULL_CONST})?
+                (\s+ {COUNT_CONST} \s+ {COUNT})?
+            )|
             (\s+ {HELP})
         )
         \s*""",
@@ -540,6 +555,12 @@ NEW_GRAMMAR = {
             (\s+ {NOLOOP_CONST})
         )*
         \s*""",
+    "command_categorynamex": fr"(\s+ {CATEGORYNAME})? \s*",
+    "command_usernames": fr"(\s+ {USERNAME})+ \s*",
+    "command_username": fr"\s+ {USERNAME} \s*",
+    "command_count_or_resetx": fr"( (\s+ {COUNT}) | (\s+ {RESET_CONST}) )? \s*",
+    "command_username_rules": fr"\s+ {USERNAME} (\s+ {RULE})* \s*",
+    "command_count": fr"(\s+ {COUNT})? \s*",
 }
 
 pipeline = r"(?P<shellcommand>\|.*)?"
@@ -551,8 +572,8 @@ def get_command_grammar(command):
     :param command: command name in upper case. This command must be raw user
         input, otherwise can't match in lexer, cause this command to be invalid;
     """
-    syntax_name = command2syntax[command.upper()]
-    syntax = NEW_GRAMMAR.get(syntax_name)
+    syntax_name = command2syntax[" ".join(command.split()).upper()]
+    syntax = GRAMMAR.get(syntax_name)
 
     # If a command is not supported yet, (e.g. command from latest version added
     # by Redis recently, or command from third Redis module.) return a defualt
